/*
 * NamedSequence.h
 *
 *  Created on: Apr 15, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#ifndef NAMEDSEQUENCE_H_
#define NAMEDSEQUENCE_H_

#include "Sequence.h"
#include <string>
using namespace std;
/*
 *
 */
namespace bio {

   class NamedSequence : virtual public Sequence {
      string name;
   protected:
      virtual void readFAHeaderLine(const string& s);
      virtual void readIGHeaderLine(const string& s);
   public:
      NamedSequence();
      NamedSequence(const NamedSequence& other);
      NamedSequence(const string& name, const string& data);
      virtual ~NamedSequence();
      using Sequence::operator=;
      NamedSequence& operator=(const NamedSequence& other) {
         Sequence::operator=(other);
         name = other.getName();
         return *this;
      }
      bool operator==(const NamedSequence& other) {
         return Sequence::operator==(other) && name==other.name;
      }
      virtual void clear() {
         Sequence::clear();
         name.clear();
      }
      const string& getName() const {
         return name;
      }
      void setName(const string& name) {
         this->name = name;
      }
      virtual void printFAHeader(ostream& os, const char* comment = NULL) const;
      virtual void printIGHeader(ostream& os, const char* comment = NULL) const;
   };

}

#endif /* NAMEDSEQUENCE_H_ */
