/*
 * NamedSubSequence.cpp
 *
 *  Created on: Apr 15, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#include "NamedSubSequence.h"
#include <sstream>
using namespace std;

namespace bio {

   NamedSubSequence::NamedSubSequence() {
   }
   NamedSubSequence::NamedSubSequence(const NamedSubSequence& other) : Sequence(other), SubSequence(other), NamedSequence(other) {
   }
   NamedSubSequence::NamedSubSequence(const NamedSequence& other) : Sequence(other), NamedSequence(other) {
   }
   NamedSubSequence::NamedSubSequence(const SubSequence& other) : Sequence(other), SubSequence(other) {
   }
   NamedSubSequence::NamedSubSequence(const string& name, const string& data) : NamedSequence(name, data) {
   }
   NamedSubSequence::~NamedSubSequence() {
   }
   void NamedSubSequence::readFAHeaderLine(const string& s) {
      SubSequence::readFAHeaderLine(s);
      NamedSequence::readFAHeaderLine(s);
   }
   void NamedSubSequence::readIGHeaderLine(const string& s) {
      SubSequence::readIGHeaderLine(s);
      NamedSequence::readIGHeaderLine(s);
   }
   void NamedSubSequence::printFAHeader(ostream& os, const char* comment) const {
      os << ">" << getName();
      for(fraglist_t::const_iterator fptr = getFragments().begin(); fptr != getFragments().end(); fptr++) {
         string sf;
         (*fptr).write(sf);
         os << "\t" << FRAGMENT << ' ' << sf;
      }
      if(comment) {
         os << "\t" << comment;
      }
      os << endl;
   }
   void NamedSubSequence::printIGHeader(ostream& os, const char* comment) const {
      if(comment) {
         os << "; " << comment << "\n";
      }
      for(fraglist_t::const_iterator fptr = getFragments().begin(); fptr != getFragments().end(); fptr++) {
         os << "; " << FRAGMENT << ' ' << *fptr << "\n";
      }
      os << "; \n" <<  getName() << endl;
   }
}
