/*
 * SubSequence.cpp
 *
 *  Created on: Apr 15, 2010
 *      Author: Oleksiy Kohany <kohany@gmail.com>
 */

#include "SubSequence.h"
#include <sstream>
using namespace std;

namespace bio {
   const char SubSequence::FRAGMENT[] = "FRAGMENT";
   SubSequence::SubSequence() {
   }
   SubSequence::SubSequence(const SubSequence& other) :
      Sequence(other), fragments(other.getFragments()) {
   }
   SubSequence::~SubSequence() {
   }
   void SubSequence::readFragments(const string& sLine, size_t pos) {
      while ((pos = sLine.find(FRAGMENT, pos)) != string::npos) {
         Fragment frag;
         pos += strlen(FRAGMENT);
         try {
            frag.read(sLine.substr(pos));
            addFragment(frag);
         }
         catch(const istream::failure& err) {
            cerr << "ERROR: Failed to read fragment record from '" << sLine << "' :" << pos << endl;
         }
      }
   }
   void SubSequence::readFAHeaderLine(const string& s) {
      readFragments(s, 1);
   }
   void SubSequence::readIGHeaderLine(const string& s) {
      if(!s.empty() && s[0] == ';') {
         readFragments(s, 1);
      }
   }
   void SubSequence::printFAHeader(ostream& os, const char* comment) const {
      os << ">ANONYMOUS";
      for(fraglist_t::const_iterator fptr = fragments.begin(); fptr != fragments.end(); fptr++) {
         os << "\t" << FRAGMENT << ' ' << *fptr;
      }
      if(comment) {
         os << "\t" << comment;
      }
      os << endl;
   }
   void SubSequence::printIGHeader(ostream& os, const char* comment) const {
      if(comment) {
         os << "; " << comment << "\n";
      }
      for(fraglist_t::const_iterator fptr = fragments.begin(); fptr != fragments.end(); fptr++) {
         os << "; " << FRAGMENT << ' ' << *fptr << "\n";
      }
      os << "; \nANONYMOUS" << endl;
   }
   bool SubSequence::isDirect() const {
      if(fragments.size() > 0) {
         return fragments.front().isDirect();
      }
      return true;
   }
   void SubSequence::setFragment(const Fragment& f) {
      fragments.clear();
      addFragment(f);
   }
   void SubSequence::setFragment(size_t b, size_t e) {
      setFragment(Fragment("", b, e));
   }
   void SubSequence::setBeg(size_t b) {
      if(fragments.size() == 0) {
         fragments.push_back(Fragment("", b, 0));
      }
      else {
         fragments.front().setBeg(b);
      }
   }
   void SubSequence::setEnd(size_t e) {
      if(fragments.size() == 0) {
         fragments.push_back(Fragment("", 0, e));
      }
      else {
         fragments.back().setEnd(e);
      }
   }
   size_t SubSequence::getBeg() {
      if(fragments.size() == 0) {
         return 0;
      }
      return fragments.front().getBeg() + 1;
   }
   size_t SubSequence::getEnd() {
      if(fragments.size() == 0) {
         return 0;
      }
      return fragments.back().getEnd();
   }
   void SubSequence::reposition(const Fragment& parent) {
      if(fragments.size() == 0) {
         fragments.push_back(parent);
      }
      else {
         fragments.front().reposition(parent);
      }
   }
}
